# AWS CloudFormation template for KIE Node using Amazon ECS on AWS Fargate

###### _Version 2.1 (Markdown instructions, this document: `kie-node-cfn.md`)_

###### _Version 2.1 (AWS CloudFormation template: `kie-node-cfn.yml`)_

This guide reviews the steps required to create a Karlsgate Identity Exchange (KIE) Node in Amazon Web Services (AWS) using Amazon ECS on AWS Fargate, Amazon EFS volume, and Amazon S3.

Please note that these instructions assume *all* the following are true:

* You have an active AWS account with the appropriate permissions to create new AWS cloud resources; 

* You have the `aws` command line interface (AWS CLI) installed on your computer and configured to access your AWS account (for more information, please refer to the [AWS Command Line Interface](https://aws.amazon.com/cli/) documenation); 

* You have the *Session Manager* plugin installed on your computer and configured for your AWS CLI installation (for more information, please refer to the [AWS Systems Manager Session Manager](https://docs.aws.amazon.com/systems-manager/latest/userguide/session-manager.html) documentation); and 

* You are logged in to the AWS Management Console for your region (e.g., https://us-east-1.console.aws.amazon.com/console/home).

## Creating a KIE Node

Follow these steps to create AWS cloud resources for a KIE Node.

1. Via the AWS Management Console, navigate to the AWS CloudFormation page for your region (e.g., https://us-east-1.console.aws.amazon.com/cloudformation). If necessary, you may search for "templates" using the AWS Management Console's search feature.

2. Press the **Create stack** button, and then choose **With new resources (standard)**.

3. On the Create stack page named Step 1, **Create stack**:

    3.1. In the *Prerequisite - Prepare template* section, choose the **Choose an existing template** option.

    3.2. In the *Specify template* section, choose **Upload a template file** option.
   
    3.3. In the *Specify template* section, use the **Upload a template file** control to find and select the `kie-node-cfn.yml` YAML template file, which accompanied this guide.
   
    3.4. Press the **Next** button.

4. On the Create stack page named Step 2, **Specify stack details**:

    4.1. Enter a **Stack name** value between 3 and 26 characters long, lowercase alphanumeric or hyphen characters only (single hyphens may separate alphanumerics) and must not begin or end with a hyphen. This is *not* known on the KIE Portal. ***IMPORTANT***: All of your AWS resources' names will be prefixed with this value. Be sure to choose a unique (globally amongst your AWS resources) and identifying value.

    4.2. Enter a **NodeNickName** parameter value, or leave the default value. This helps identify this KIE Node on the KIE Portal and must be unique within your KIE membership (that is, a KIE membership cannot have 2+ active nodes with the same node nickname).

    4.3. Enter a **PublicSubnetOneCIDR** parameter value, or leave the default value. This is the IP range [CIDR] for the public subnet in the first Availability Zone.

    4.4. Enter a **PublicSubnetTwoCIDR** parameter value, or leave the default value. This is the IP range [CIDR] for the public subnet in the second Availability Zone.

    4.5. Enter a **VpcCIDR** parameter value, or leave the default value. This is the IP range [CIDR] for your VPC and depends on IP ranges available in your account.

    4.6 Press the **Next** button.

5. On the Create stack page named Step 3, **Configure stack options**:

    5.1. Review the values and either modify them as required for your AWS implementation or leave the defaults.

    5.2. Check the **I acknowledge that AWS CloudFormation might create IAM resources with custom names** checkbox.

    5.3. Press the **Next** button.

6. On the Create stack page named  Step 4, **Review and create**:

    6.1. Review the summarized values.

    6.2. Press the **Submit** button.

Note: The AWS cloud resources created using this template may take a few minutes (or longer) to be completely provisioned. Wait until this stack is *complete* before proceeding with the subsequent steps below.

## Verifying the ECS resource

Follow these steps to verify that your AWS cloud resources have been provisioned.

7. Via the AWS Management Console, navigate to the Amazon Elastic Container Service page for your region (e.g., https://us-east-1.console.aws.amazon.com/ecs). If necessary, you may search for "ecs" using the AWS Management Console's search feature.

8. Find the cluster you created using this template (the cluster's name will begin with the value you entered for *ResourceNamePrefix* in step 4.5 [above]), and verify the cluster's task is running. *IMPORTANT: Please make note of your ECS cluster's name.*

    Note: Now that you have verified that your ECS is running, you may activate your KIE Node. A node only needs to be activated a single time, since the activation action links it to your KIE membership.

## Activating a KIE Node

Follow these steps to activate your KIE Node by connecting to the ECS task host and its ECS task (that is, the running Docker instance).

9. On your computer that has the `aws` command line interface (AWS CLI) installed and configured to access your AWS account, open the AWS CLI.

10. Using the AWS CLI, execute the following command to list the running tasks:

    `aws ecs list-tasks --cluster {ecs-cluster-name}`

    ...where `{ecs-cluster-name}` is the name of your ECS cluster.  Please make note of the Region Code and Task ID displayed (see example output below); you will also still need your ECS cluster name.

    The output of this command will be in the format:

    ```json
    {
      "taskArns": [
         "arn:aws:ecs:{region-code}:{account-id}:task/{ecs-cluster-name}/{task-id}"
      ]
    }
    ```

11. Using the AWS CLI, execute the following command to connect to the ECS cluster:

    `aws ecs execute-command --region {region-code} --cluster {ecs-cluster-name} --task {task-id} --command "/bin/sh" --interactive`

    ...where `{region-code}` is the Region Code, `{ecs-cluster-name}` is the ECS cluster name, and `{task-id}` is the Task ID from the `taskArns` value returned in the previous step.

    Note: Once you have successfully connected to your KIE Node using the AWS CLI & SessionManager plugin, your AWS CLI prompt will change.

12. Using the AWS CLI (when you have successfully connected to the ECS task), execute the following command (and then answer the subsequent on-screen questions) to activate your KIE Node:

    `kie node activate`

    Note: For more information on the activate command, read: https://portal.karlsgate.com/Help/Nodes#activate-node

    12.1. Next, enter the **User Name** (email address) for a KIE Portal user assigned the *Node Manager* user role. *IMPORTANT: This user must be assigned the **Node Manager** user role on the KIE Portal.*

    12.2. Then, enter the passcode for the user specified in the previous step, which will either be:

    * an **Auth Code** (used for single sign-on configurations),

    * a **One-Time Code** (if provided to you), or

    * a **Password** (most commonly).

    12.3. Lastly, at the **Would you like to change the default storage settings?** prompt, enter: `n` (for "no").

    Note #1: This KIE Node is now activated. (A node will only be activated *once*.)

    Note #2: Activating a node will result in two (2) folders being created in the S3 bucket associated with this node. These folders are: `output/` and `staging/`. For more information on staging data via this `staging/` folder, refer to the [staging data](https://portal.karlsgate.com/Help/Automation#staging-data) documentation. Any trade-related output received by this node will be located in subfolders of the `output/` folder.

13. Returning to the AWS CLI, verify your KIE Node's configuration by executing the command:

    `kie node diagnose`

    Note: This command peforms a series of diagnostic tests to verify the node's environment, networking, and connected storage. Should the diagnostic reveal any off-nominal states and/or issues, which you are unable to resolve yourself, do not hesitate to contact: [help@karlsgate.com](mailto:help@karlsgate.com)

14. To exit the Session Manager, execute the following command in the AWS CLI:

    `exit`

To reconnect to your activated KIE Node in the future, repeat Steps 9 through 11 (above). However, *instead* of performing Steps 12 & 13, execute your desired node command(s). Finally, `exit` (Step 14) the Session Manager once you have executed your commands.

## Decommisioning (Disabling) a KIE Node

Should you decide to decommission and disable a KIE Node, the best practice is to fully clean-up the KIE Node and its associated resources before deprovisioning the cloud resources. This will require connecting to the node via the AWS CLI (refer to Steps 9 through 11 plus 14, above).

* First (optionally), remove every listings associated with the node by executing the `kie listings remove`*`{segmenthandle}`* command for each listing (e.g., `kie listings remove sample`). (To display a node's listings, execute: `kie listings show`)

* Second, permanently disable the node by executing the `kie node disable` command, and confirm the node's disablement. *Note: This operation cannot be undone.*

* Next, permanently delete *all* files and folders (aka. objects) from the node's S3 connected storage, so that the S3 is empty. If applicable, save any necessary files beforehand. (This ensures that you intentionally deleted your data.)

* Lastly, delete the stack, which will deprovision the AWS resources that were provisioned via the AWS CloudFormation template (e.g., https://us-east-1.console.aws.amazon.com/cloudformation/home).

---

<style type="text/css">
    ul { list-style-type: upper-alpha; }
</style>
